#include "Panel_free_drawing.h"

#include "Path.h"
#include "Default_builder.h"
#include "Atom_kit.h"
#include "Bond_kit.h"
#include "Visual_style.h"
//#include "Geometry.h"
#include "Cleaner.h"
#include "Point_3D_impl.h"
#include "Measure_geometry_impl.h"
#include "Atom_numerator.h"
#include "Molecular_detector.h"

#include <QMouseEvent>

namespace MM
{
double Panel_free_drawing:: min_distance = 0.5;

Panel_free_drawing::
Panel_free_drawing ()
:
    Panel ("Draw", "right"),
    fixed_atom_ (0),
    moving_atom_(0)
{
    setupUi (this);

    toolButton_Chain        ->hide();

    label_MM_type           ->hide();
    comboBox_MM_type        ->hide();
    label_Fine_type         ->hide();
    comboBox_Fine_type      ->hide();
    pushButton_Try_to_assign->hide();

    //====================================================


    connect(toolButton_C,   SIGNAL (toggled (bool)),
            this,           SLOT   (set_C   (bool)));

    connect(toolButton_O,   SIGNAL (toggled (bool)),
            this,           SLOT   (set_O   (bool)));

    connect(toolButton_N,   SIGNAL (toggled (bool)),
            this,           SLOT   (set_N   (bool)));

    connect(toolButton_H,   SIGNAL (toggled (bool)),
            this,           SLOT   (set_H   (bool)));

    toolButton_C->setChecked (true);

    for (int i=0;  i<Element::count();  ++i)
        listWidget->addItem (Element(i).c_str());

    connect(listWidget,     SIGNAL (itemClicked (QListWidgetItem *)),
            this,           SLOT   (set_element (QListWidgetItem *)));
    

    toolButton_CH4    ->setIcon(QIcon(Path::kernels() + "CH4.png"));
    toolButton_H2O    ->setIcon(QIcon(Path::kernels() + "H2O.png"));
    toolButton_NH3    ->setIcon(QIcon(Path::kernels() + "NH3.png"));
    toolButton_H2S    ->setIcon(QIcon(Path::kernels() + "H2S.png"));
    toolButton_C2H4   ->setIcon(QIcon(Path::kernels() + "C2H4.png"));
    toolButton_HCHO   ->setIcon(QIcon(Path::kernels() + "HCHO.png"));
    toolButton_Furan  ->setIcon(QIcon(Path::kernels() + "Furan.png"));
    toolButton_Benzene->setIcon(QIcon(Path::kernels() + "Benzene.png"));

    connect(toolButton_CH4, SIGNAL (clicked ()),
            this,           SLOT   (CH4_kernel ()));

    connect(toolButton_H2O, SIGNAL (clicked ()),
            this,           SLOT   (H2O_kernel ()));

    connect(toolButton_NH3, SIGNAL (clicked ()),
            this,           SLOT   (NH3_kernel ()));

    connect(toolButton_H2S, SIGNAL (clicked ()),
            this,           SLOT   (H2S_kernel ()));

    connect(toolButton_C2H4, SIGNAL (clicked ()),
            this,            SLOT   (C2H4_kernel ()));

    connect(toolButton_HCHO, SIGNAL (clicked ()),
            this,            SLOT   (HCHO_kernel ()));

    connect(toolButton_Furan, SIGNAL (clicked ()),
            this,             SLOT   (Furan_kernel ()));

    connect(toolButton_Benzene, SIGNAL (clicked ()),
            this,               SLOT   (Benzene_kernel ()));



    connect(comboBox_Bond,  SIGNAL (currentIndexChanged (const QString &)),
            this,           SLOT   (set_bond_order      (const QString &)));

    connect(pushButton_Remove_hydrogen,  SIGNAL (clicked ()),
            this,           SLOT   (remove_hydrogen ()));

    connect(pushButton_Add_hydrogen,  SIGNAL (clicked ()),
            this,           SLOT   (add_hydrogen ()));

    connect_handlers ();
}

Panel_free_drawing::
~Panel_free_drawing ()
{
    disconnect_handlers ();
}

void Panel_free_drawing::
set_bond_order (const QString & text)
{
    order_ = Order (Text(text).c_str());
}

void Panel_free_drawing::
set_element (QListWidgetItem * item)
{
    element_ = Text(item->text()).c_str();

    if (element_ == "C")
    {
        toolButton_C->blockSignals(true);
        toolButton_C->setChecked (true);
        toolButton_C->blockSignals(false);
    }
    else if (element_ == "O")
    {
        toolButton_O->blockSignals(true);
        toolButton_O->setChecked (true);
        toolButton_O->blockSignals(false);
    }
    else if (element_ == "N")
    {
        toolButton_N->blockSignals(true);
        toolButton_N->setChecked (true);
        toolButton_N->blockSignals(false);
    }
    else if (element_ == "H")
    {
        toolButton_H->blockSignals(true);
        toolButton_H->setChecked (true);
        toolButton_H->blockSignals(false);
    }
    else
    {
        toolButton_Chain->setChecked (true);

        //toolButton_C->blockSignals(true);
        //toolButton_O->blockSignals(true);
        //toolButton_N->blockSignals(true);
        //toolButton_H->blockSignals(true);

        ////toolButton_C->setChecked (false);
        ////toolButton_O->setChecked (false);
        ////toolButton_N->setChecked (false);
        ////toolButton_H->setChecked (false);

        //if (toolButton_C->isChecked ())
        //    toolButton_C->toggle ();
        //if (toolButton_O->isChecked ())
        //    toolButton_O->toggle ();
        //if (toolButton_N->isChecked ())
        //    toolButton_N->toggle ();
        //if (toolButton_H->isChecked ())
        //    toolButton_H->toggle ();

        //toolButton_C->blockSignals(false);
        //toolButton_O->blockSignals(false);
        //toolButton_N->blockSignals(false);
        //toolButton_H->blockSignals(false);
    }
}

void Panel_free_drawing::
set_C (bool on)
{
    if (on)
    {
        element_ = "C";
        listWidget->clearSelection ();
    }
}

void Panel_free_drawing::
set_O (bool on)
{
    if (on)
    {
        element_ = "O";
        listWidget->clearSelection ();
    }
}

void Panel_free_drawing::
set_N (bool on)
{
    if (on)
    {
        element_ = "N";
        listWidget->clearSelection ();
    }
}

void Panel_free_drawing::
set_H (bool on)
{
    if (on)
    {
        element_ = "H";
        listWidget->clearSelection ();
    }
}

void Panel_free_drawing::
CH4_kernel ()
{
    Text file_name = model().kit().file_name ();
    model().load (Path::kernels() + "CH4.mlm");
    model().kit().set_file_name (file_name);
    set_ball_and_stick ();
}

void Panel_free_drawing::
H2O_kernel ()
{
    Text file_name = model().kit().file_name ();
    model().load (Path::kernels() + "H2O.mlm");
    model().kit().set_file_name (file_name);
    set_ball_and_stick ();
}

void Panel_free_drawing::
NH3_kernel ()
{
    Text file_name = model().kit().file_name ();
    model().load (Path::kernels() + "NH3.mlm");
    model().kit().set_file_name (file_name);
    set_ball_and_stick ();
}

void Panel_free_drawing::
H2S_kernel ()
{
    Text file_name = model().kit().file_name ();
    model().load (Path::kernels() + "H2S.mlm");
    model().kit().set_file_name (file_name);
    set_ball_and_stick ();
}

void Panel_free_drawing::
C2H4_kernel ()
{
    Text file_name = model().kit().file_name ();
    model().load (Path::kernels() + "C2H4.mlm");
    model().kit().set_file_name (file_name);
    set_ball_and_stick ();
}

void Panel_free_drawing::
HCHO_kernel ()
{
    Text file_name = model().kit().file_name ();
    model().load (Path::kernels() + "HCHO.mlm");
    model().kit().set_file_name (file_name);
    set_ball_and_stick ();
}

void Panel_free_drawing::
Furan_kernel ()
{
    Text file_name = model().kit().file_name ();
    model().load (Path::kernels() + "Furan.mlm");
    model().kit().set_file_name (file_name);
    set_ball_and_stick ();
}

void Panel_free_drawing::
Benzene_kernel ()
{
    Text file_name = model().kit().file_name ();
    model().load (Path::kernels() + "Benzene.mlm");
    model().kit().set_file_name (file_name);
    set_ball_and_stick ();
}


void Panel_free_drawing::
remove_hydrogen ()
{
    Model & current_model = model ();
    int     count = current_model.atom_count ();
    Element H("H");
    Cleaner clear(current_model);

    for (int i=0;  i<count;  ++i)
    {
        Atom & atom = current_model.atom (i);

        if (atom.element() == H)
            clear.add (atom);
    }
}

void Panel_free_drawing::
add_hydrogen ()
{
    static Element C("C"), O("O"), N("N"), S("S");
    Model & current_model = model ();
    int     count = current_model.atom_count ();

    for (int i=0;  i<count;  ++i)
    {
        Atom & atom = current_model.atom (i);
        int current_valency = valency (atom);
        
        if (atom.element() == C)
            add_H (atom, 4 - current_valency);
        
        else if (atom.element() == N)
            add_H (atom, 3 - current_valency);
        
        else if (atom.element() == O || atom.element() == S)
            add_H (atom, 2 - current_valency);
    }
}

void Panel_free_drawing::
add_H (Atom & atom, int n)
{
    //static Element H("H");
    Default_builder         builder;
    Measure_geometry_impl   geometry;

    switch (n)
    {
        case 1:
        {
            int count = atom.bound_count();

            //if (count == 1)
            //{
            //    Atom & H1 = builder.new_atom (model(), H);
            //
            //    Point_3D const & position = atom.bound(0).position();
            //    Vector_3D_impl vector(0,0,0);
            //    base.difference (position, &vector);
            //    result += vector.norm ();
            //    H1.set_position (result);
            //    if (vector.dot(Vector_3D_impl(0,0,1)) < 0.9)
            //        H1.rotate (Vector_3D_impl(0,0,1), base, 3.14 * 80. / 180.);
            //    else
            //        H1.rotate (Vector_3D_impl(0,1,0), base, 3.14 * 80. / 180.);
            //    
            //    H1.kit().visual().set_style (Visual_style::ball_and_stick);
            //    Bond & bond = builder.new_bond (model(), atom, H1, Order());
            //    bond.kit().visual().set_style (Visual_style::ball_and_stick);
            //}
            //else 
            if (count >= 1)
            {
                model().add (Path::kernels() + "H2O.mlm");
                Molecule & H2O = model().molecule (model().molecule_count() - 1);
                Atom & O  = H2O.atom(0);
                Atom & H1 = H2O.atom(1);
                Atom & H2 = H2O.atom(2);

                H1.kit().visual().set_style (Visual_style::ball_and_stick);

                Atom_numerator numerator(model());

                geometry.set_atom_type (0, atom.kit().number());
                geometry.set_atom_type (1, O.kit().number());

                Length_unit distance;
                distance.set_A (0.);
                geometry.set_parameter (distance);

                geometry.set_atom_type (0, atom.bound(0).kit().number());
                geometry.set_atom_type (1, atom.kit().number());            //fix why?
                geometry.set_atom_type (2, H2.  kit().number());
                    
                Angle_unit angle;
                angle.set_rad (0.);
                geometry.set_parameter (angle); //fix H2O + -1 +

                if (count > 1)
                {
                    Vector_3D_impl sum(0,0,0);
                    Point_3D const & base = atom.position();
                    Point_3D_impl result (base);

                    for (int i=1;  i<count;  ++i)
                    {
                        Vector_3D_impl vector(0,0,0);
                        Point_3D const & position = atom.bound(i).position();
                        base.difference (position, &vector);
                        sum += vector.norm ();
                    }

                    Vector_3D_impl cross (0,0,0);
                    Vector_3D_impl first (0,0,0);
                    Vector_3D_impl second(0,0,0);
                    base.difference (atom.bound(0).position(), &first);
                    base.difference (atom.bound(1).position(), &second);
                    first.cross (second, &cross);

                    if (sum.norm().dot (cross.norm()) < 0.1 && count == 3)
                        result += cross;
                    else
                        result += sum;

                    geometry.set_space_position_type (0, result);
                    geometry.set_atom_type (1, H2.kit().number());
                    geometry.set_atom_type (2, O. kit().number());          
                    geometry.set_atom_type (3, H1.kit().number());

                    angle.set_rad (0.);
                    geometry.set_parameter (angle);
                }

                Cleaner cleaner(model());
                cleaner.add (O);
                cleaner.add (H2);
                cleaner.execute();

                Bond & bond_1 = builder.new_bond (model(), atom, H1, Order());
                bond_1.kit().visual().set_style (Visual_style::ball_and_stick);
            }
            break;
        }

        case 2:
        {
            if (atom.bound_count() == 0)
            {
                model().add (Path::kernels() + "H2O.mlm");
                Molecule & H2O = model().molecule (model().molecule_count() - 1);
                Atom & O  = H2O.atom(0);
                Atom & H1 = H2O.atom(1);
                Atom & H2 = H2O.atom(2);

                H1.kit().visual().set_style (Visual_style::ball_and_stick);
                H2.kit().visual().set_style (Visual_style::ball_and_stick);

                Atom_numerator numerator(model());

                geometry.set_atom_type (0, atom.kit().number());
                geometry.set_atom_type (1, O.kit().number());

                Length_unit distance;
                distance.set_A (0.);
                geometry.set_parameter (distance);

                Cleaner cleaner(model());
                cleaner.add (O);
                cleaner.execute();

                Bond & bond_1 = builder.new_bond (model(), atom, H1, Order());
                bond_1.kit().visual().set_style (Visual_style::ball_and_stick);
                Bond & bond_2 = builder.new_bond (model(), atom, H2, Order());
                bond_2.kit().visual().set_style (Visual_style::ball_and_stick);
            }
            else if (atom.bound_count() == 1)
            {
                if (atom.bond(0).order() == Order::Single)
                {
                    model().add (Path::kernels() + "CH4.mlm");
                    Molecule & CH4 = model().molecule (model().molecule_count() - 1);
                    Atom & C  = CH4.atom(0);
                    Atom & H1 = CH4.atom(1);
                    Atom & H2 = CH4.atom(2);
                    Atom & H3 = CH4.atom(3);
                    Atom & H4 = CH4.atom(4);

                    H1.kit().visual().set_style (Visual_style::ball_and_stick);
                    H2.kit().visual().set_style (Visual_style::ball_and_stick);

                    Atom_numerator numerator(model());

                    geometry.set_atom_type (0, atom.kit().number());
                    geometry.set_atom_type (1, C.kit().number());

                    Length_unit distance;
                    distance.set_A (0.);
                    geometry.set_parameter (distance);

                    geometry.set_atom_type (0, atom.bound(0).kit().number());
                    geometry.set_atom_type (1, atom.kit().number());            //fix why?
                    geometry.set_atom_type (2, H4.  kit().number());
                    
                    Angle_unit angle;
                    angle.set_rad (0.);
                    geometry.set_parameter (angle);

                    Cleaner cleaner(model());
                    cleaner.add (C);
                    cleaner.add (H3);
                    cleaner.add (H4);
                    cleaner.execute();

                    Bond & bond_1 = builder.new_bond (model(), atom, H1, Order());
                    bond_1.kit().visual().set_style (Visual_style::ball_and_stick);
                    Bond & bond_2 = builder.new_bond (model(), atom, H2, Order());
                    bond_2.kit().visual().set_style (Visual_style::ball_and_stick);
                }
                else // Double or Aromatic
                {
                    model().add (Path::kernels() + "H2O.mlm");
                    Molecule & H2O = model().molecule (model().molecule_count() - 1);
                    Atom & O  = H2O.atom(0);
                    Atom & H1 = H2O.atom(1);
                    Atom & H2 = H2O.atom(2);

                    H1.kit().visual().set_style (Visual_style::ball_and_stick);
                    H2.kit().visual().set_style (Visual_style::ball_and_stick);

                    Atom_numerator numerator(model());

                    geometry.set_atom_type (0, atom.bound(0).kit().number());
                    geometry.set_atom_type (1, atom.kit().number());
                    geometry.set_mass_center_type 
                        (2, model().molecule_count() - 1);

                    Angle_unit angle;
                    angle.set_deg (180.);
                    geometry.set_parameter (angle);

                    geometry.set_atom_type (0, atom.kit().number());
                    geometry.set_atom_type (1, O.kit().number());

                    angle.set_deg (180.);
                    geometry.set_parameter (angle);

                    if (atom.bound(0).bound_count() > 1)
                    {
                        int atom_0_N;
                        if (&atom.bound(0).bound(0) == &atom)
                            atom_0_N = atom.bound(0).bound(1).kit().number();
                        else
                            atom_0_N = atom.bound(0).bound(0).kit().number();

                        geometry.set_atom_type (0, atom_0_N);
                        geometry.set_atom_type (1, atom.bound(0).kit().number());
                        geometry.set_atom_type (2, O.kit().number());
                        geometry.set_atom_type (3, H1.kit().number());

                        angle.set_deg (0.);
                        geometry.set_parameter (angle);
                    }

                    geometry.set_atom_type (1, atom.kit().number());
                    geometry.set_atom_type (2, O.kit().number());
                    geometry.set_active (0, false);
                    geometry.set_active (3, false);

                    Length_unit distance;
                    distance.set_A (0.);
                    geometry.set_parameter (distance);

                    Cleaner cleaner(model());
                    cleaner.add (O);
                    cleaner.execute();

                    Bond & bond_1 = builder.new_bond (model(), atom, H1, Order());
                    bond_1.kit().visual().set_style (Visual_style::ball_and_stick);
                    Bond & bond_2 = builder.new_bond (model(), atom, H2, Order());
                    bond_2.kit().visual().set_style (Visual_style::ball_and_stick);
                }
            }
            else if (atom.bound_count() == 2)
            {
                model().add (Path::kernels() + "CH4.mlm");
                Molecule & CH4 = model().molecule (model().molecule_count() - 1);
                Atom & C  = CH4.atom(0);
                Atom & H1 = CH4.atom(1);
                Atom & H2 = CH4.atom(2);
                Atom & H3 = CH4.atom(3);
                Atom & H4 = CH4.atom(4);

                H1.kit().visual().set_style (Visual_style::ball_and_stick);
                H2.kit().visual().set_style (Visual_style::ball_and_stick);

                Atom_numerator numerator(model());

                geometry.set_atom_type (0, atom.kit().number());
                geometry.set_atom_type (1, C.kit().number());

                Length_unit distance;
                distance.set_A (0.);
                geometry.set_parameter (distance);

                geometry.set_atom_type (0, atom.bound(0).kit().number());
                geometry.set_atom_type (1, atom.kit().number());            //fix why?
                geometry.set_atom_type (2, H4.  kit().number());
                
                Angle_unit angle;
                angle.set_rad (0.);
                geometry.set_parameter (angle);

                geometry.set_atom_type (0, atom.bound(1).kit().number());
                geometry.set_atom_type (1, H4.kit().number());
                geometry.set_atom_type (2, C. kit().number());
                geometry.set_atom_type (3, H3.kit().number());

                angle.set_deg (0.);
                geometry.set_parameter (angle);

                Cleaner cleaner(model());
                cleaner.add (C);
                cleaner.add (H3);
                cleaner.add (H4);
                cleaner.execute();

                Bond & bond_1 = builder.new_bond (model(), atom, H1, Order());
                bond_1.kit().visual().set_style (Visual_style::ball_and_stick);
                Bond & bond_2 = builder.new_bond (model(), atom, H2, Order());
                bond_2.kit().visual().set_style (Visual_style::ball_and_stick);
            }

            break;
        }
        case 3:
        {
            if (atom.bound_count() != 1)
                break;

            //for (int i=0;  i<model().atom_count();  ++i)
            //    double x = model().atom(i).x();

            model().add (Path::kernels() + "CH4.mlm");
            Molecule & CH4 = model().molecule (model().molecule_count() - 1);
            Atom & C  = CH4.atom(0);
            Atom & H1 = CH4.atom(1);
            Atom & H2 = CH4.atom(2);
            Atom & H3 = CH4.atom(3);
            Atom & H4 = CH4.atom(4);

            //for (int i=0;  i<model().atom_count();  ++i)
            //    double x = model().atom(i).x();

            H1.kit().visual().set_style (Visual_style::ball_and_stick);
            H2.kit().visual().set_style (Visual_style::ball_and_stick);
            H3.kit().visual().set_style (Visual_style::ball_and_stick);

            Atom_numerator numerator(model());

            geometry.set_atom_type (0, atom.kit().number());
            geometry.set_atom_type (1, C.kit().number());

            Length_unit distance;
            distance.set_A (0.);
            geometry.set_parameter (distance);

            geometry.set_atom_type (0, atom.bound(0).kit().number());
            geometry.set_atom_type (1, atom.kit().number());            //fix why?
            geometry.set_atom_type (2, H4.  kit().number());
            
            Angle_unit angle;
            angle.set_rad (0.);
            geometry.set_parameter (angle);

            Cleaner cleaner(model());
            cleaner.add (C);
            cleaner.add (H4);
            cleaner.execute();

            Bond & bond_1 = builder.new_bond (model(), atom, H1, Order());
            bond_1.kit().visual().set_style (Visual_style::ball_and_stick);
            Bond & bond_2 = builder.new_bond (model(), atom, H2, Order());
            bond_2.kit().visual().set_style (Visual_style::ball_and_stick);
            Bond & bond_3 = builder.new_bond (model(), atom, H3, Order());
            bond_3.kit().visual().set_style (Visual_style::ball_and_stick);

            break;
        }
    }

    Molecular_detector detect(model());
    detect.execute();
}

int Panel_free_drawing::
valency (Atom & atom)
{
    static Order 
        Single   (Order::Single),
        Aromatic (Order::Aromatic),
        Double   (Order::Double),
        Triple   (Order::Triple);

    double result = 0.;

    for (int i=0;  i<atom.bound_count();  ++i)
    {
        Bond & bond = atom.bond (i);
        Order order = bond.order();

        if (order == Single)
            result += 1.0;

        else if (order == Aromatic)
            result += 1.5;

        else if (order == Double)
            result += 2.0;

        else if (order == Triple)
            result += 3.0;

        else                    //fix
            result += 1.0;
    }

    return int (result+0.6);
}

void Panel_free_drawing::
showEvent (QShowEvent * event)
{
    //if (!isVisible ())
    {
        //connect_handlers ();
    }

    QWidget::showEvent (event);
}

void Panel_free_drawing::
hideEvent (QHideEvent * event)
{
    if (!isVisible ())
    {
        //disconnect_handlers ();
    }

    QWidget::hideEvent (event);
}

void Panel_free_drawing::
connect_handlers ()
{
    REQUIRE ("Only one additional event handler.", 
        Model_RM_widget::press_         == 0 &&
        Model_RM_widget::release_       == 0 &&
        Model_RM_widget::double_click_  == 0 &&
        Model_RM_widget::move_          == 0
        );

    Model_RM_widget::press_         = &press_;
    Model_RM_widget::release_       = &release_;
    Model_RM_widget::double_click_  = &double_click_;
    Model_RM_widget::move_          = &move_;
}

void Panel_free_drawing::
disconnect_handlers ()
{
    Model_RM_widget::press_        =0;
    Model_RM_widget::release_      =0;
    Model_RM_widget::double_click_ =0;
    Model_RM_widget::move_         =0;
}

void Panel_free_drawing::
position (QMouseEvent * event, Model_RM_widget * rm_widget,
          double &x, double &y, double &z,
          RMvertex3D & eye, RMvertex3D & at)
{
    double dx, dy, dz;
    RMvertex3D eye_vector, up_vector, rigth_vector, at_point;
    int dX = event->x() - rm_widget->width  () / 2;
    int dY = event->y() - rm_widget->height () / 2;

    rm_widget->movement (event, dX, dY,
        &dx, &dy, &dz, 
        &eye_vector, &up_vector, &rigth_vector, &at_point);

    x = -at_point.x - dx;
    y = -at_point.y - dy;
    z = -at_point.z - dz;

    at  = at_point;
    eye = eye_vector;
}

Atom * Panel_free_drawing::
cross_atom (Model & model, double x, double y, double z, RMvertex3D const& eye)
{
    REQUIRE ("", moving_atom_ != 0);

    RMvertex3D moving_position;
    moving_position.x = x;
    moving_position.y = y;
    moving_position.z = z;

    for (int i=0;  i<model.atom_count();  ++i)
    {
        Atom & atom = model.atom (i);

        if (&atom == moving_atom_)
            continue;

        RMvertex3D atom_position, w, l;
        atom_position.x = atom.x();
        atom_position.y = atom.y();
        atom_position.z = atom.z();
        rmVertex3DDiff  (&atom_position, &moving_position, &w);
        rmVertex3DCross (const_cast <RMvertex3D*> (&eye), &w, &l);
        double distance = rmVertex3DMag (&l);

        if (distance < min_distance)
            return &atom; 
    }

    return 0;
}

bool Panel_free_drawing::
Mouse_press::execute (QMouseEvent *event, Model_RM_widget * rm_widget)
{
    Panel_free_drawing * self = Panel_free_drawing::singleton();
    return self->mouse_press (event, rm_widget);
}

bool Panel_free_drawing::
mouse_press (QMouseEvent * event, Model_RM_widget *rm_widget)
{
    //if (event->buttons () & Qt::RightButton)
    if (event->buttons () & Qt::LeftButton)
    {
        rm_widget->model().kit().selector().clear_picking ();

        //if (event->modifiers() == Qt::ControlModifier)
        //{
        //    if (pick_atom   (event, rm_widget))
        //        delete_atom (event, rm_widget);

        //    else if (pick_bond (event, rm_widget))
        //        delete_bond    (event, rm_widget);
        //}
        //else
        if (event->modifiers() == (Qt::ShiftModifier | Qt::ControlModifier))
        {
            if (pick_atom  (event, rm_widget))
                clone_atom (event, rm_widget);

            else if (pick_bond (event, rm_widget))
                set_bond       (event, rm_widget);

            else
                new_atom (event, rm_widget);
        }
        return true;
    }

    return false;
}

bool Panel_free_drawing::
Mouse_release::execute (QMouseEvent *event, Model_RM_widget * rm_widget)
{
    Panel_free_drawing * self = Panel_free_drawing::singleton();
    return self->mouse_release (event, rm_widget);
}


bool Panel_free_drawing::
mouse_release (QMouseEvent * event, Model_RM_widget *rm_widget)
{
    //if (event->buttons () & Qt::RightButton)
    {
        if (event->modifiers() == Qt::ControlModifier ||
            fixed_atom_ == 0)
        {
            fixed_atom_ = 0;
            return false;
        }

        double x, y, z;
        RMvertex3D eye, at;
        position (event, rm_widget, x, y, z, eye, at);

        Atom * crossed = cross_atom (rm_widget->model(), x, y, z, eye);
        if (crossed != 0/* && moving_atom_ !=0*/)
        {
            Model & model = rm_widget->model();
            
            Cleaner cleaner(model);
            cleaner.add (*moving_atom_);
            cleaner.execute ();
            moving_atom_ = 0;

            if (crossed != fixed_atom_)
            {
                Default_builder builder;
                Bond & bond = builder.new_bond  
                    (model, *fixed_atom_, *crossed, order_);
                bond.kit().visual().set_style (Visual_style::ball_and_stick);
                
                rm_widget->model().kit().selector().pick (bond);
            }
            else    // change element
            {
                fixed_atom_->set_element (element_);
                fixed_atom_->set_charge  (doubleSpinBox_Charge->value());
                rm_widget->model().kit().selector().pick (*fixed_atom_);
            }
        }
        else
            rm_widget->model().kit().selector().pick (*moving_atom_);

        fixed_atom_ = 0;

        Molecular_detector detect(model());
        detect.execute();

        return true;
    }

    return false;
}

bool Panel_free_drawing::
pick_atom (QMouseEvent * event, Model_RM_widget * rm_widget)
{
    rm_widget->pick (event->x(), event->y());
    return rm_widget->model().kit().selector().pick_count() == 1;
}

bool Panel_free_drawing::
pick_bond (QMouseEvent * event, Model_RM_widget * rm_widget)
{
    rm_widget->pick (event->x(), event->y());
    return rm_widget->model().kit().selector().pick_count() > 1;
}

void Panel_free_drawing::
clone_atom (QMouseEvent *, Model_RM_widget * rm_widget)
{
    REQUIRE ("There is picked atom", 
        rm_widget->model().kit().selector().pick_count() == 1);

    Default_builder builder;
    fixed_atom_ = &rm_widget->model().kit().selector().last_picked();

    if (pushButton_Clone->isChecked ())
    {
        moving_atom_ = fixed_atom_->clone();
        rm_widget->model().add_atom (moving_atom_);
    }
    else
    {
        Default_builder builder;
        moving_atom_ = &builder.new_atom (rm_widget->model(), element_);
        moving_atom_->set_charge (doubleSpinBox_Charge->value());

        moving_atom_->set_x (fixed_atom_->x());
        moving_atom_->set_y (fixed_atom_->y());
        moving_atom_->set_z (fixed_atom_->z());
        moving_atom_->kit().visual().set_style (Visual_style::ball_and_stick);
    }

    Bond & bond  = builder.new_bond  (rm_widget->model(), 
        *fixed_atom_, *moving_atom_, order_);

    fixed_atom_ ->kit().visual().set_style (Visual_style::ball_and_stick);
    moving_atom_->kit().visual().set_style (Visual_style::ball_and_stick);
    bond.         kit().visual().set_style (Visual_style::ball_and_stick);
}

void Panel_free_drawing::
new_atom (QMouseEvent * event, Model_RM_widget * rm_widget)
{
    Default_builder builder;
    fixed_atom_  = &builder.new_atom (rm_widget->model(), element_);
    moving_atom_ = &builder.new_atom (rm_widget->model(), element_);
    Bond & bond  = builder.new_bond  (rm_widget->model(), 
        *fixed_atom_, *moving_atom_, order_);

    double x, y, z;
    RMvertex3D eye, at;
    position (event, rm_widget, x, y, z, eye, at);

    fixed_atom_ ->set_x (x);
    fixed_atom_ ->set_y (y);
    fixed_atom_ ->set_z (z);
    fixed_atom_ ->set_charge (doubleSpinBox_Charge->value());
    moving_atom_->set_x (x);
    moving_atom_->set_y (y);
    moving_atom_->set_z (z);
    moving_atom_->set_charge (doubleSpinBox_Charge->value());

    fixed_atom_ ->kit().visual().set_style (Visual_style::ball_and_stick);
    moving_atom_->kit().visual().set_style (Visual_style::ball_and_stick);
    bond.         kit().visual().set_style (Visual_style::ball_and_stick);
}

void Panel_free_drawing::
set_bond (QMouseEvent *, Model_RM_widget * rm_widget)
{
    Atom & atom_1 = rm_widget->model().kit().selector().last_picked();
    Bond * bond   = 0;

    for (int i=0;  i<atom_1.bound_count();  ++i)
    {
        if (atom_1.bound(i).kit().is_picked())
        {
            bond = &atom_1.bond(i);
            break;
        }
    }
    CHECK ("", bond != 0);

    bond->set_order (order_);
    rm_widget->model().kit().selector().pick (*bond);
}

bool Panel_free_drawing::
Mouse_double_click::execute (QMouseEvent *event, Model_RM_widget * rm_widget)
{
    Panel_free_drawing * self = Panel_free_drawing::singleton();
    return self->mouse_double_click (event, rm_widget);
}

bool Panel_free_drawing::
mouse_double_click (QMouseEvent * event, Model_RM_widget * /*rm_widget*/)
{
    if (event->buttons () & Qt::RightButton)
    {
        return true;
    }
    return false;
}

bool Panel_free_drawing::
Mouse_move::execute (QMouseEvent *event, Model_RM_widget * rm_widget)
{
    Panel_free_drawing * self = Panel_free_drawing::singleton();
    return self->mouse_move (event, rm_widget);
}

bool Panel_free_drawing::
mouse_move (QMouseEvent * event, Model_RM_widget *rm_widget)
{
    //if (event->buttons () & Qt::RightButton)
    if ((event->buttons () & Qt::LeftButton) && 
        (event->modifiers() == (Qt::ShiftModifier | Qt::ControlModifier)))
    {
        if (moving_atom_ != 0)
        {
            double x, y, z;
            RMvertex3D eye, at;
            position (event, rm_widget, x, y, z, eye, at);

            moving_atom_->set_x (x);
            moving_atom_->set_y (y);
            moving_atom_->set_z (z);

            Atom * crossed = cross_atom (rm_widget->model(), x, y, z, eye);
            if (crossed != 0)
                rm_widget->model().kit().selector().pick (*crossed);
        }

        return true;
    }

    return false;
}

void Panel_free_drawing::
delete_atom (QMouseEvent *, Model_RM_widget *rm_widget)
{
    REQUIRE ("There is picked atom", 
        rm_widget->model().kit().selector().pick_count() == 1);

    Model & model = rm_widget->model();
    Atom  & atom  = model.kit().selector().last_picked();
    Cleaner cleaner(model);
    cleaner.add (atom);
    cleaner.execute ();

    Molecular_detector(model).execute();
}

void Panel_free_drawing::
delete_bond (QMouseEvent *, Model_RM_widget *rm_widget)
{
    REQUIRE ("There is picked bond", 
        rm_widget->model().kit().selector().pick_count() > 1);

    Model & model = rm_widget->model();
    
    Atom     & atom_1   = model.kit().selector().last_picked();
    Selector & selector = model.kit().selector();
    selector.set_max_picked_atoms (1);
    selector.pop();
    Atom     & atom_2   = model.kit().selector().last_picked();

    int bond_n;
    atom_1.bound_with (atom_2, &bond_n);
    Bond & bond = atom_1.bond (bond_n);

    Cleaner cleaner(model);
    cleaner.add (bond);
    cleaner.execute ();

    Molecular_detector(model).execute();
}

void Panel_free_drawing::
set_ball_and_stick ()
{
    Model & m = model ();
    int i, count = m.atom_count ();

    for (i=0;  i<count;  ++i)
        m.atom(i).kit().visual().set_style (Visual_style::ball_and_stick);

    count = m.bond_count ();

    for (i=0;  i<count;  ++i)
        m.bond(i).kit().visual().set_style (Visual_style::ball_and_stick);
}

}//MM
