#ifndef PANEL_FREE_DRAWING_H
#define PANEL_FREE_DRAWING_H

#ifndef PANEL_H
#include "Panel.h"
#endif

#ifndef DYNAMIC_SINGLETON_H
#include "Dynamic_singleton.h"
#endif

#ifndef UI_PANEL_FREE_DRAWING_H
#include "ui_Panel_free_drawing.h"
#endif

#ifndef MODEL_RM_WIDGET_H
#include "Model_RM_widget.h"
#endif

#ifndef ELEMENT_H
#include "Element.h"
#endif

#ifndef ORDER_H
#include "Order.h"
#endif

namespace MM
{

class Panel_free_drawing
:
    public Panel,
    public Ui_Panel_free_drawing,
    public Dynamic_singleton <Panel_free_drawing>
{
    Q_OBJECT

    static double               min_distance;
    Element                     element_;
    Order                       order_;
    Atom *                      fixed_atom_;
    Atom *                      moving_atom_;

public:    
    Panel_free_drawing ();
   ~Panel_free_drawing ();

public slots:
    void    set_bond_order (const QString &  text);
    void    set_element    (QListWidgetItem * item);
    void    set_C          (bool on);
    void    set_O          (bool on);
    void    set_N          (bool on);
    void    set_H          (bool on);

    void    CH4_kernel     ();
    void    H2O_kernel     ();
    void    NH3_kernel     ();
    void    H2S_kernel     ();
    void    C2H4_kernel    ();
    void    HCHO_kernel    ();
    void    Furan_kernel   ();
    void    Benzene_kernel ();

    void    remove_hydrogen ();
    void    add_hydrogen    ();

protected:
    void    showEvent (QShowEvent * event);
    void    hideEvent (QHideEvent * event);

    void    connect_handlers ();
    void    disconnect_handlers ();

    void    position (QMouseEvent * event, Model_RM_widget * rm_widget,
                      double &x, double &y, double &z,
                      RMvertex3D & eye, RMvertex3D & at);

    Atom *  cross_atom (Model & model, double x, double y, double z, 
                        RMvertex3D const& eye);


public:    
    struct Mouse_press : Model_RM_widget::Mouse_handler 
    {virtual bool execute (QMouseEvent * event, Model_RM_widget *);} press_;
    bool    mouse_press   (QMouseEvent * event, Model_RM_widget *);

    struct Mouse_release : Model_RM_widget::Mouse_handler 
    {virtual bool execute (QMouseEvent * event, Model_RM_widget *);} release_;
    bool    mouse_release (QMouseEvent * event, Model_RM_widget *);

    struct Mouse_double_click : Model_RM_widget::Mouse_handler 
    {virtual bool execute (QMouseEvent * event, Model_RM_widget *);} double_click_;
    bool    mouse_double_click (QMouseEvent * event, Model_RM_widget *);

    struct Mouse_move : Model_RM_widget::Mouse_handler 
    {virtual bool execute (QMouseEvent * event, Model_RM_widget *);} move_;
    bool    mouse_move    (QMouseEvent * event, Model_RM_widget *);

protected:
    bool    pick_atom    (QMouseEvent *, Model_RM_widget *);
    bool    pick_bond    (QMouseEvent *, Model_RM_widget *);
    void    clone_atom   (QMouseEvent *, Model_RM_widget *);
    void    new_atom     (QMouseEvent *, Model_RM_widget *);
    void    set_bond     (QMouseEvent *, Model_RM_widget *);
    void    delete_atom  (QMouseEvent *, Model_RM_widget *);
    void    delete_bond  (QMouseEvent *, Model_RM_widget *);

    int     valency (Atom &);   //fix move
    void    add_H (Atom &, int);

    void    set_ball_and_stick ();
};

}//MM

#endif //PANEL_FREE_DRAWING_H
