#include "Panel_NWChem.h"

#include "Log.h"

#include "Defs.h"
#include "Path.h"
#include "User.h"
#include "Model.h"
#include "Model_kit.h"
#include "Atom.h"
#include "Main_window.h"

#include <QCloseEvent>
#include <QDir>

#include <sstream>
#include <fstream>
#include <math.h>

namespace MM
{
using namespace std;

Panel_NWChem::
Panel_NWChem ()
:
    Panel ("NWChem", "left"),

    model_                  (0),
    process_                (new QProcess(this)),
    output_                 (setup_NWChem_dir() + "NWChem-output"),
    //output_                 (Path::report() + "NWChem-output"),
    player_                 (Main_window::singleton()),
    read_atom_coords_flag_  (false),
    read_charges_flag_      (false),
    read_moments_flag_      (false),
    user_stop_flag_         (false),
    success_                (false),
    optimization_it_        (0)
{
    setupUi (this);

    //====================================================

    //listWidget_Basis->item(5)->setHidden (true);

    //====================================================


    connect (comboBox_RUNTYP,   SIGNAL(currentIndexChanged (const QString &)),
             this,              SLOT  (show_options        (const QString &)));

    process_->setReadChannel (QProcess::StandardOutput);
    process_->setWorkingDirectory (NWChem_dir());

    connect (process_,          SIGNAL(started()),
             this,              SLOT  (started()));

    connect (process_,          SIGNAL(finished (int, QProcess::ExitStatus)),
             this,              SLOT  (finished (int, QProcess::ExitStatus)));

    connect (process_,          SIGNAL(readyRead()),
             this,              SLOT  (read()));

    listWidget_Basis->setCurrentRow (4);
}

Panel_NWChem::
~Panel_NWChem ()
{
    if (process_->isOpen ())
        process_->kill ();

    if (output_.isOpen ())
    {
        output_.flush ();
        output_.close ();
    }
}

void Panel_NWChem::
turn (bool on)
{
    //Main_window *player = Main_window::singleton();

    //if (process_->state() == QProcess::NotRunning)
    if (!player_->player_on_)
    {
        player_->toolButton_Stop   -> disconnect ();
        player_->toolButton_Back   -> disconnect ();
        player_->toolButton_Tune   -> disconnect ();
        player_->toolButton_Run    -> disconnect ();
        player_->toolButton_Start  -> disconnect ();
        player_->toolButton_Forward-> disconnect ();
        player_->toolButton_Pause  -> disconnect ();

        if (on)
        {
            player_->tabWidget_bottom->setCurrentWidget (player_->tab_Run);

            connect (player_->toolButton_Stop,   SIGNAL(clicked()), this,
                                 SLOT  (stop   ()));
            connect (player_->toolButton_Tune,   SIGNAL(clicked()), this,
                                 SLOT  (tune   ()));
            connect (player_->toolButton_Run,    SIGNAL(clicked()), this,
                                 SLOT  (run    ()));
            connect (player_->toolButton_Start,  SIGNAL(clicked()), this,
                                 SLOT  (start  ()));
            set_stop_state ();
        }
        else 
        {
            player_->toolButton_Stop   -> setEnabled (false);
            player_->toolButton_Back   -> setEnabled (false);
            player_->toolButton_Tune   -> setEnabled (false);
            player_->toolButton_Run    -> setEnabled (false);
            player_->toolButton_Start  -> setEnabled (false);
            player_->toolButton_Forward-> setEnabled (false);
            player_->toolButton_Pause  -> setEnabled (false);
        }
    }
}

void Panel_NWChem::
closeEvent (QCloseEvent * event)
{
    if (process_->state() == QProcess::Running)
        event->ignore ();
}

Text const & Panel_NWChem::
setup_NWChem_dir ()
{
    NWChem_dir_ = Path::NWChem_bin();
    //ifstream path_file ((NWChem_dir_ + "path.txt").c_str());

    //if (path_file)
    //{
    //    string new_path;
    //    getline (path_file, new_path);
    //    CP2K_dir_ = new_path.c_str();
    //}

    return NWChem_dir_;
}

void Panel_NWChem::
show_options (const QString & text)
{
    for (int i=0;  i<stackedWidget->count();  ++i)
    {
        if (stackedWidget->widget(i)->objectName() == text)
            stackedWidget->setCurrentIndex (i);
    }
}

void Panel_NWChem::
run ()
{
    if (process_->state() == QProcess::NotRunning)
    {
        tune  ();
        start ();
    }
    else
        stop();
}

void Panel_NWChem::
tune ()
{
    model_ = &model();

    create_input ();
    to_user().status ("NWChem input generated.");
}

void Panel_NWChem::
start ()
{
    model_ = &model();

    if (process_->state() == QProcess::NotRunning)
    {
        to_user().info ("\n___________________________________________");

        if (output_.isOpen ())
            output_.close ();

        if (!output_.open (QIODevice::WriteOnly/* | QIODevice::Text*/))
        {
            to_user().error ("Can not open 'output' file.");     
            return;
        }

        QString name = stackedWidget->currentWidget()->objectName();//fix to state
        if (name == "Optimize")
        {
            optimization_it_ = 1;
        }
        else
            optimization_it_ = 0;

        QDir dir(NWChem_dir());
        dir.remove ("nwchem.b");
        dir.remove ("nwchem.b");
        dir.remove ("nwchem.b^-1");
        dir.remove ("nwchem.c");
        dir.remove ("nwchem.db");
        dir.remove ("nwchem.drv.hess");
        dir.remove ("nwchem.grid");
        dir.remove ("nwchem.movecs");
        dir.remove ("nwchem.mp2nos");
        dir.remove ("nwchem.p");
        dir.remove ("nwchem.plt");
        dir.remove ("nwchem.q");
        dir.remove ("nwchem.t2");
        dir.remove ("nwchem.t2.numg");
        dir.remove ("nwchem.xyz");
        dir.remove ("nwchem.zmat");

        Text        program = NWChem_dir() +"nw32.exe";
        QStringList arguments;
        arguments << "nwchem.nw";

        QStringList environment = QProcess::systemEnvironment();
        environment << ("HOME=" + Path::NWChem());
        environment << ("NWCHEM_BASIS_LIBRARY=" + Path::NWChem() + 
            "usr.local.lib.nwchem\\libraries\\"); 
        environment << ("NWCHEM_NWPW_LIBRARY="  + Path::NWChem() + 
            "usr.local.lib.nwchem\\libraryps\\");  
        environment.replaceInStrings
            (QRegExp("^PATH=(.*)", Qt::CaseInsensitive), "PATH=\\1;" + 
            Path::NWChem_bin());
        process_->setEnvironment (environment);   

        user_stop_flag_ = false;

        process_->start (program, arguments);

        bool started = process_->waitForStarted (3000);
        if (!started)
        {
            to_user().fatal_error (
                "Can not start NWChem.\n\n"
                "Please check that there are \"NWChem\" "
                "in Ascalaph/NWChem folder.\n"
                );
            set_stop_state ();
        }
    }
    else
        stop();
}

void Panel_NWChem::
stop ()
{
//    pushButton_Check->setEnabled (false);
    user_stop_flag_ = true;
    player_->player_on_ = false;
    process_->kill();

    to_user ().info ("Simulation was stopped by the user.");
}

void Panel_NWChem::
started ()
{
    to_user ().status ("NWChem started");
    set_run_state ();
}

void Panel_NWChem::
finished (int, QProcess::ExitStatus exitStatus)
{
    //qApp->processEvents(QEventLoop::ExcludeSocketNotifiers);

    if (exitStatus == QProcess::CrashExit && !user_stop_flag_)
    {
        Text message ("NWChem crashed. ");
        to_user ().status (message);

        switch (process_->error ())
        {
        case QProcess::FailedToStart:
            to_user ().error (message +
                "The process failed to start. "
                "Either the invoked program is missing, or you may have "
                "insufficient permissions to invoke the program.");
            break;

        case QProcess::Crashed:
            to_user ().error (message +
                "The process crashed some time after "
                "starting successfully.");
            break;

        case QProcess::Timedout:
            to_user ().error (message + "Timed out");
            break;

        case QProcess::WriteError:
            to_user ().error (message +
                "An error occurred when attempting to write to the process.");
            break;

        case QProcess::ReadError:
            to_user ().error (message +
                "An error occurred when attempting to read from "
                "the process. For example, the process may not be running.");
            break;

        case QProcess::UnknownError:
            to_user ().error (message + "An unknown error occurred.");
            break;
        }
    }
    else
    {
        //read ();
        output_.flush ();

        Text message ("NWChem finished ");
        if (success_)
            message += "NORMALLY";
        //else
        //    to_user ().warning ("Execution of CP2K terminated ABNORMALLY.");

        to_user ().status (message);
    }

    read ();
    set_stop_state ();
}

void Panel_NWChem::
set_stop_state ()
{
    player_->player_on_ = false;

    player_->toolButton_Stop   -> setEnabled (false);
    player_->toolButton_Tune   -> setEnabled (true);
    player_->toolButton_Run    -> setEnabled (true);
    player_->toolButton_Start  -> setEnabled (true);

    if (this->isHidden ())
        turn (false);
}

void Panel_NWChem::
set_run_state ()
{
    player_->player_on_ = true;

    player_->toolButton_Stop   -> setEnabled (true);
    player_->toolButton_Tune   -> setEnabled (false);
    player_->toolButton_Run    -> setEnabled (false);
    player_->toolButton_Start  -> setEnabled (false);
}

void Panel_NWChem::
read ()
{
    char    buf[1024];
    //Text    text;
    char *  result = 0;
    string  stub;
    //Text    message;

    while (process_->canReadLine ())
    {
        qint64 bytes = process_->readLine (buf, sizeof(buf));
        output_.write (buf, bytes);

        for (int i=0;  i<1024;  ++i)
        {
            if (buf[i] == '\0')
            {
                buf[i-2] = 0x0A;
                buf[i-2] = '\0';
            }
        }

        if (read_atom_coords_flag_)
        {
            read_atom_coords (buf);
            continue;
        }

        if (read_charges_flag_)
        {
            read_charges (buf);
            continue;
        }

        if (read_moments_flag_)
        {
            read_moments (buf);
            continue;
        }

        //=====================================================================
        qApp->processEvents (QEventLoop::ExcludeSocketNotifiers);

        result = strstr (buf, "Output coordinates");
        if (result)
        {
            read_atom_coords_flag_ = true;
            continue;
        }

        result = strstr (buf, " ESP");
        if (result)
        {
            to_user().info ("");
            read_charges_flag_ = true;
            read_charges (buf);
            continue;
        }

        result = strstr (buf, "Dipole moment");
        if (result &&
            !strstr (buf, "Nuclear Dipole moment"))
        {
            to_user().info ("");
            //to_user().info ("Dipole moment [Debye]");
            to_user().info (buf);
            //read_moments_flag_ = true;
            continue;
        }

        double e;

        result = strstr (buf, "Total SCF energy");
        if (result)
        {
            if (optimization_it_!=0)
                continue;

            istringstream in (buf);
            //int    it;
            in >> stub >> stub >> stub >> stub >> e;

            Text message;
            message += Text (" E        = ");
            message += Text (e, 15);
            message += " Hartree   ";
            message += Text (e*Constant::Hartree_to_kcal, 15);
            message += " kcal/mol";
            to_user ().info (message);
            message.clear ();

            continue;
        }

        result = strstr (buf, "Total MP2 energy");
        if (result)
        {
            if (optimization_it_!=0)
                continue;

            istringstream in (buf);
            in >> stub >> stub >> stub >> e;

            Text message;
            message += Text (" E (MP2)  = ");
            message += Text (e, 15);
            message += " Hartree   ";
            message += Text (e*Constant::Hartree_to_kcal, 15);
            message += " kcal/mol";
            to_user ().info (message);
            message.clear ();

            continue;
        }


        result = strstr (buf, "Total CCSD energy:");
        if (result)
        {
            if (optimization_it_!=0)
                continue;

            istringstream in (buf);
            in >> stub >> stub >> stub >> e;

            Text message;
            message += Text (" E (CCSD) = ");
            message += Text (e, 15);
            message += " Hartree   ";
            message += Text (e*Constant::Hartree_to_kcal, 15);
            message += " kcal/mol";
            to_user ().info (message);
            message.clear ();

            continue;
        }

        result = strstr (buf, "Total CCSD(T) energy:");
        if (result)
        {
            if (optimization_it_!=0)
                continue;

            istringstream in (buf);
            in >> stub >> stub >> stub >> e;

            Text message;
            message += Text (" E CCSD(T) = ");
            message += Text (e, 15);
            message += " Hartree   ";
            message += Text (e*Constant::Hartree_to_kcal, 15);
            message += " kcal/mol";
            to_user ().info (message);
            message.clear ();

            continue;
        }

        //if (message.is_not_empty())
        //{
        //    to_user ().info (message);
        //    message.clear ();
        //}

        //=====================================================================
        char *abnormal;

        abnormal= 
            strstr (buf, "EXECUTION OF NWChem TERMINATED ABNORMALLY");
        if (abnormal)
            to_user ().fatal_error (buf);

        abnormal = strstr (buf, "*** ERROR");
        if (abnormal || strstr (buf, "ERROR!"))
            to_user ().error (buf);

        char *normal = strstr (buf, "EXECUTION OF NWChem TERMINATED NORMALLY");
        if (normal)
            success_ = true;
    }
    //log () << text;
    output_.flush ();
//    qApp->processEvents (QEventLoop::ExcludeSocketNotifiers);
}

void Panel_NWChem::
read_atom_coords (char *buf)
{
    static int n = -3;
    string stub;
    double x, y, z;

    int length = strlen (buf);
    if (n > 0 && length < 4)
    {
        read_atom_coords_flag_ = false;
        n = -3;
        model_->kit().event().conformation_changed ();
        return;
    }

    if (n >=0)
    {
        Atom & atom = model_->atom (n);

        istringstream in (buf);
        in >> stub >> stub >> stub >> x >> y >> z;
        atom.set_x (x);
        atom.set_y (y);
        atom.set_z (z);
    }

    ++n;
}

void Panel_NWChem::
read_charges (char *buf)
{
    static int n = -3;
    string stub;
    double charge;

    if (n >= 0 && strlen(buf) > 50 && buf[50] == '-')
    {
        read_charges_flag_ = false;
        n = -3;
        return;
    }

    if (n >=0)
    {
        Atom & atom = model_->atom (n);

        istringstream in (buf);
        in >> stub >> stub >> stub >> stub >> stub >> charge;
        atom.set_charge (charge);

        to_user().info (buf);
    }

    ++n;
}

void Panel_NWChem::
read_moments (char *buf)
{
    int length = strlen (buf);
    if (length <= 2)
    {
        read_moments_flag_ = false;
        return;
    }

    to_user().info (buf);
}

void Panel_NWChem::
create_input ()
{
    success_ = false;
    Text job = stackedWidget->currentWidget()->objectName();
    Text basis_name (listWidget_Basis->currentItem()->text());


    Text text;
    ofstream out ((NWChem_dir() + "nwchem.nw").c_str());

    out << "title \"" << model().kit().file_name().c_str() << " " 
        << basis_name.c_str() << " " << job.c_str() << "\"" << endl;
    out << endl;

    out << "charge " << spinBox_Charge->value() << endl;
    out << endl;

    out << "geometry "      << endl;
    out << data().c_str();
    out << "end "           << endl;
    out << endl;
    
    out << "basis " << spherical_or_cartesian().c_str() << endl;
    out << " * library " << basis_name.c_str() << endl;
    out << "end "       << endl;
    out << endl;

    if (comboBox_RUNTYP->currentText() == "Electrostatics")
    {
        out << "esp "     << endl;
        out << " recalculate "     << endl;
        out << " probe   " << doubleSpinBox_probe  ->value() / 10. << endl;
        out << " range   " << doubleSpinBox_range  ->value() / 10. << endl;
        out << " factor  " << doubleSpinBox_factor ->value()       << endl;
        out << " spacing " << doubleSpinBox_spacing->value() / 10. << endl;
        out << "end "       << endl;
        out << endl;
    }

    if (comboBox_RUNTYP->currentText() == "Optimize")
    {
        out << "driver "     << endl;
        out << " clear "     << endl;
        out << " "          << quality().c_str() << endl;
        out << " eprec   "  << Text(lineEdit_Gradient->text()).c_str() << endl;
        out << " maxiter "  << Text(spinBox_NSTEP    ->text()).c_str() << endl;
        out << "end "       << endl;
        out << endl;
    }

    if (comboBox_Calc->currentText() != "Fast" /*&& comboBox_Task->currentText().contains("mp2")*/)
    {
        out << "mp2 "               << endl;
        out << " tight "            << endl;
        out << " freeze atomic "    << endl;
        out << "end "               << endl;
        out << endl;
    }
    
    
    out << "task " << Text(comboBox_Task->currentText()).c_str() << " " << operation().c_str() << endl;

    //fix 
    if (comboBox_RUNTYP->currentText() == "Electrostatics")
        out << "task esp" << endl;

    out << endl;
    out << endl;
    return;
}

Text Panel_NWChem::
basis ()
{
    Text text;

    return text;
}

Text Panel_NWChem::
data ()
{
    Text text;

    for (int i=0;  i<model_->atom_count();  ++i)
    {
        Atom const & atom = model_->atom(i);

        text += " ";
        text += atom.element().c_str();
        text += "    ";
        text += Text(atom.x(), 15);
        text += " ";
        text += Text(atom.y(), 15);
        text += " ";
        text += Text(atom.z(), 15);
        text += "   # ";
        text += i + 1;
        text += "\n";
    }

    return text;
}

Text Panel_NWChem::
operation ()
{
    if (comboBox_RUNTYP->currentText() == "Energy")
        return "energy";

    if (comboBox_RUNTYP->currentText() == "Electrostatics")
        //return "property";
        return "energy";

    if (comboBox_RUNTYP->currentText() == "Optimize")
        return "optimize";
}

Text Panel_NWChem::
quality ()
{
    if (comboBox_Calc->currentText() == "Normal")
        return "DEFAULT";

    if (comboBox_Calc->currentText() == "Accurate")
        return "TIGHT";

    if (comboBox_Calc->currentText() == "Fast")
        return "LOOSE";
}

bool Panel_NWChem::
not_semiempirical ()
{
    //Text basis = comboBox_Basis->currentText();
    //Text panel = tabWidget_Basis->currentWidget()->objectName();

    //if (panel == "Assembly" &&
    //    (basis == "MNDO" || basis == "AM1" || basis == "PM3" || basis == "RM1"))
    //    return false;

    return true;
}

Text Panel_NWChem::
spherical_or_cartesian () const
{
    if (listWidget_Basis->currentItem()->text().contains("-cc-"))
        return "spherical";

    return "cartesian";
}

}//MM
